#ifndef MSP_GL_VERTEXARRAY_H_
#define MSP_GL_VERTEXARRAY_H_

#include <climits>
#include <vector>
#include <msp/core/refptr.h>
#include <msp/datafile/loader.h>
#include "bufferable.h"
#include "datatype.h"
#include "primitivetype.h"
#include "vertexarraybuilder.h"
#include "vertexformat.h"

namespace Msp {
namespace GL {

class Buffer;

/**
Stores vertex data.

The array's contents can be modified with the append and modify methods.  To
obtain the location of an individual component within the vertex, use
VertexFormat::offset.

A higher-level interface for filling in vertex data is available in the
VertexArrayBuilder class.
*/
class VertexArray: public Bufferable
{
public:
	class Loader: public DataFile::Loader, public VertexArrayBuilder
	{
	public:
		Loader(VertexArray &);
	};

private:
	VertexFormat format;
	std::vector<float> data;
	unsigned stride;

	VertexArray(const VertexArray &);
	VertexArray &operator=(const VertexArray &);
public:
	VertexArray(const VertexFormat &);

	/// Resets the VertexArray to a different format.  All data is cleared.
	void reset(const VertexFormat &);

	const VertexFormat &get_format() const { return format; }

	/// Clears all vertices from the array.
	void clear();

	/// Reserve space for vertices.
	void reserve(unsigned);

	/// Append a new vertex at the end of the array and return its location.
	float *append();

	/// Returns the location of a vertex for modification.
	float *modify(unsigned);
private:
	virtual unsigned get_data_size() const;
	virtual const void *get_data_pointer() const { return &data[0]; }

public:
	unsigned size() const { return data.size()/stride; }
	const std::vector<float> &get_data() const { return data; }
	const float *operator[](unsigned i) const { return &data[0]+i*stride; }
};

} // namespace GL
} // namespace Msp

#endif
